<?php

/*
 $Source: /tmp/maintain/modules/class/domain.php,v $

 Copyright (c) 2004 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 * domain.php - defines the @see Domain class 
 * @version $Id: domain.php,v 1.32 2004/05/27 19:38:36 philips Exp $
 * @copyright Copyright (c) 2004 Oregon State University - Network Engineering
 * @filesource
 */

/**
 * Domain - allows for updating, modifying and deleting a domain
 *
 * @package network_model
 */

class Domain {
	
	/**
	 * Domains id number - used as unique id in the database
	 * @var		integer
	 * @access	public
	 */
	var $id;
	/**
	 * A string representing the domains name ex. bookstore.mu.oregonstate.edu
	 * @var		string
	 * @access	public
	 */
	var $name;
	/**
	 * Id number of the master domain
	 * @var		integer
	 * @access	public
	 */
	var $master_domain;
	/**
	 * History of the domain
	 * @var		object @see ObjectHistory
	 * @access	public
	 */
	var $object_history;
	/**
	 * True - domain enabled False - Domain Disabled
	 * @var		bool
	 * @access	private
	 */
	var $_enabled;
	
	// we can populate this stuff from the database -- no sense in
	//   passing it in.
	function Domain ( $domain_id ) {
		global $conf;

		$this->id = $domain_id;

		// Grab all information about the domain from the database
		if ($domain_id) {
			$info = $this->get_info();
		}

		if ( ! $info ) {
			return false;
		}

		// set all of the objects attributes
		// Mapped 1 to 1 from the database
		$this->name		     	= $info->name;
		$this->master_domain 	= $info->master_domain;
		$this->_enabled			= $info->enabled;
		
		$this->object_history	= new ObjectHistory('domain', $this->id);
	} // Domain

	/**
	*
	* Gets all of the info from the domain table
	*
	* @return	resource	Returns a mysql result set object
	* @access	public
	* @see		mysql_fetch_object()
	*/
	function get_info () {
		global $conf;

		$sql = "SELECT * FROM domain WHERE id='$this->id'";
		$db_result = mysql_query($sql, dbh());
		
		return (mysql_fetch_object($db_result));

	} // get_info

	/**
	*
	* If this domain is delegated it returns all nameservers.
	* Else, it returns an empty array.
	*
	* @return	array	@see NS
	* @access	public
	*/
	function get_ns () {
		global $conf;

		$sql = "SELECT id FROM nameserver WHERE domain='$this->id'";
		$db_result = mysql_query($sql, dbh());

		$ns_arr = array();

		while ( $ns_id = mysql_fetch_object($db_result) ) {
			$ns = new NS($ns_id->id);
			$ns_arr[] = $ns;
		}

		return $ns_arr;
	}

	/**
	*
	* Gets the domain ID from a name
	*
	* This is the long description for the Class
	*
	* @return	int	 	Domain ID
	* @access	public
	* @param	$name	name of the domain we are searching for
	*/
	function get_domain_by_name($name) { 

		$sql = "SELECT id FROM domain WHERE name='$name'";
		$db_results = mysql_query($sql, dbh());

		$results = mysql_fetch_object($db_results);

		return $results->id;

	} // get_domain_by_name

	/**
	*
	* Get all hosts in this domain
	*
	* @return	array 	Returns an array of Host objects
	* @access	public
	* @see		Host
	*/
	function get_hosts() {
		global $conf;

		$sql = "SELECT * FROM host WHERE domain='$this->id'";
		$db_results = mysql_query($sql, $conf['dbh']);
	
		while ($r = mysql_fetch_object($db_results)) {
			$host = new Host($r->id);
			$hosts[] = $host;
		}
		return $hosts;

	} //get_hosts

	/**
	*
	* Deletes a domain
	*
	* A few sideffects of this action include
	* - The Domains "object history" is deleted
	* - The Domain is added to the add_delete_history
	*
	* @access	public
	* @param	$domain_id	Oddity of this method is that it requires a 
	*						domain id. 
	*/
	function delete_domain($domain_id) {
		global $conf, $user;
		
		$domain = new Domain($domain_id);
		
		// Delete the SOA for this domain
		$sql = "SELECT id FROM soa WHERE domain='$domain_id'";
		$db_results = mysql_query($sql, dbh());	
		if ($r = mysql_fetch_object($db_results)) {
			$soa = new SOA($r->id);
			$soa->delete();
		}
		
		// Delete the object
		$sql = "DELETE FROM domain WHERE id='$domain_id'";
		$db_results = mysql_query($sql, dbh());


		// Delete any object_history
		$sql = "DELETE FROM object_history WHERE type='domain' AND id='$domain_id'";
		$db_results = mysql_query($sql, dbh());

		//Add the deletion record
		$text = "Deleted Domain " . $domain->name;
		$conf['ui']->add_delete_history('0',$text,$user->username);

	} // delete

	/**
	*
	* Creates a new domain 
	*
	* The new domain is only has a master and a name,  
	*
	* @return	Integer	 Domain ID of the newly created domain
	* @access	public
	*/
	function new_domain($name, $master_domain = 0) {
		
		$sql = "INSERT INTO domain (name,master_domain,enabled) VALUES ('$name', $master_domain, 1)";
		$db_results = mysql_query($sql, dbh());
		$domain_id = mysql_insert_id(dbh());	

		return $domain_id;
		
	} // new_domain


	/**
	*
	* Get all domains that exist 
	*
	* @return	array		Returns a alphabetically ordered (by name) 
	*						array of @see Domain objects.
	* @access	public
	*/
	function domains() {
		global $user, $conf;
		
		$sql = "SELECT * FROM domain ORDER BY name";
		$db_results = mysql_query($sql, $conf['dbh']);
		
		while ($d = mysql_fetch_object($db_results)) {
			$domain = new Domain($d->id);
			$domains[] = $domain;
		}

		return $domains;
	} //domains
	
	/**
	*
	* Get all zones in this domain
	*
	* @return	array		Returns an array of @see Zone objects
	*						that this domain is a member of.
	* @access	public
	*/
	function zones() {
		global $user, $conf;

		$sql = "SELECT * FROM zone_domain WHERE domain='$this->id'";
		$db_results = mysql_query($sql, $conf['dbh']);
		
		while ($z = mysql_fetch_object($db_results))
		{
			$zones[] = new Zone($z->zone);
		}
		return $zones;

	} //zones

	
	/**
	*
	* Checks to see if the current domain belongs to $zone_id 
	*
	* @return	boolean		True if the domain belongs to the zone, false if not.
	* @access	public
	*/
	function check_zone($zone_id) {
		
		global $user, $conf;

		$sql = "SELECT * FROM zone_domain WHERE zone='$zone_id' AND domain='$this->id'";
		$db_results = mysql_query($sql, $conf['dbh']); 
		
		if (!mysql_fetch_object($db_results)) { return true; }
		else { return false; }
	}

	/**
	*
	* Checks $name and ensures it is valid 
	*
	* $domain_id is used to contraint the known owner of $name.
	* NOTE: in essence we are doing a check of uniqueness, something that 
	* should probably be done on the DB side as a FK.
	* Side Effects: 
	* 	- If duplicate the function inserts an error using $conf['error']
	*	- If domain name is invalid it inserts an error using $conf['error']
	*
	* @return	null
	* @access	public
	*/
	function check_name($name,$domain_id) {
		global $conf;

		//first lets make sure it isn't a duplicate
		$sql = "SELECT id FROM domain WHERE name LIKE '$name' AND id!='$domain_id'";
		$db_results = mysql_query($sql, dbh()); 
		if (mysql_fetch_row($db_results)) {
			$conf['error']->add_error('domain_name', "Duplicate name: $name is already in use."); 
		}

		if(!Domain::is_valid_name($name)) {	
			$conf['error']->add_error('domain_name', "$name is an invalid Domain Name"); 
		}
	
	} //check_name

	
	/**
	*
	* Updates the domain name 
	*
	* Does some checking to maintain some database integrity @see check_name
	*
	* @access	public
	*/
	function update_name($name) {
		global $conf, $user;

		$this->check_name($name, $this->id);
		
		$sql = "UPDATE domain SET name='$name' WHERE id='$this->id'";
		$db_results = mysql_query($sql, dbh());

	} // update_name

	/**
	*
	* Sets the domains enabled status
	*
	* @access	public
	*/
	function set_enabled($enabled) {
        $sql = "UPDATE domain SET enabled='$enabled' WHERE id='$this->id'";
        $db_results = mysql_query($sql, dbh());
		$this->_enabled = $enabled;
	}

	//
	
	/**
	*
	* @return	boolean	 Returns true if the Domain is enabled
	* @access	public
	*/
	function is_enabled () {
		return $this->_enabled;
	}

	/**
	*
	* Returns the status of a domain as a human readable string 
	*
	* @return	string		Disabled or Enabled.
	* @access	public
	*/
	function get_enabled_string () {
		return	UI::show_status($this->_enabled);
	}


////////////////////////////////////////////////////////////////////////////// 
// 
// Static Methods
// 
//////////////////////////////////////////////////////////////////////////////

	/**
	*
	* Verify a domain name is valid.
	*
	* @param	string	$name		domain name to verify	
	* @param	boolean	$allow_tld	allow $name to be a top level domain
	* @return	boolean		true if the name is valid
	* @access	public
	* @static
	*/
	function is_valid_name($name = null, $allow_tld = false) {
		//TODO: http://php.oregonstate.edu/manual/en/pcre.pattern.modifiers.php
		// put newlines in these expressions.
		// PROBLEM: Requires PHP 4.3

		if (preg_match("/^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?$/", $name)) {
			return true;			
		} elseif (preg_match("/^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)*[a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?$/", $name) AND $allow_tld) {
			return true;
		} else {
			return false;
		}
	}

	//
	// Debugging Methods 
	//

	// Accessor functions -- things only used by this class or for
	//    debugging purposes

	function _show () {
		print("Domain: id -> $this->id, name -> $this->name end Domain<br>\n");
	}

	function _debug () {
		if ( $this->debug ) {
			$this->debug = false;
		}
		else {
			$this->debug = true;
		}
	}
} // class Domain

?>
